@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: axis.c,v 1.97.2.2 2012/09/26 23:05:56 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: axis.c,v 1.97.2.5 2013/08/08 06:45:31 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - axis.c */
@@ -461,7 +461,7 @@ copy_or_invent_formatstring(AXIS_INDEX axis)
 	/* The simple case: formatstring is usable, so use it! */
 	strcpy(ticfmt[axis], axis_array[axis].formatstring);
 	/* Ensure enough precision to distinguish tics */
-	if (!strcmp(ticfmt[axis],"% g")) {
+	if (!strcmp(ticfmt[axis], DEF_FORMAT)) {
 	    double axmin = AXIS_DE_LOG_VALUE(axis,axis_array[axis].min);
 	    double axmax = AXIS_DE_LOG_VALUE(axis,axis_array[axis].max);
 	    int precision = (ceil(-log10(fabs(axmax-axmin))));
@@ -881,6 +881,7 @@ gen_tics(AXIS_INDEX axis, tic_callback callback)
 
 	for (mark = def->def.user; mark; mark = mark->next) {
 	    char label[MAX_ID_LEN];
+	    char *ticlabel;
 	    double internal;
 
 	    /* This condition is only possible if we are in polar mode */
@@ -893,16 +894,23 @@ gen_tics(AXIS_INDEX axis, tic_callback callback)
 	    if (!inrange(internal, internal_min, internal_max))
 		continue;
 
-	    if (mark->level < 0) /* label read from data file */
-		strncpy(label, mark->label, sizeof(label));
-	    else if (axis_array[axis].datatype == DT_TIMEDATE)
+	    if (mark->level < 0) {
+		/* label read from data file */
+		ticlabel = mark->label;
+	    } else if (mark->label && !strchr(mark->label, '%')) {
+		/* string constant that contains no format keys */
+		ticlabel = mark->label;
+	    } else if (axis_array[axis].datatype == DT_TIMEDATE) {
 		gstrftime(label, MAX_ID_LEN-1, mark->label ? mark->label : ticfmt[axis], mark->position);
-	    else
+		ticlabel = label;
+	    } else {
 		gprintf(label, sizeof(label), mark->label ? mark->label : ticfmt[axis], log10_base, mark->position);
+		ticlabel = label;
+	    }
 
 	    /* use NULL instead of label for minitic */
 	    (*callback) (axis, internal,
-	    		(mark->level>0)?NULL:label,
+	    		(mark->level>0)?NULL:ticlabel,
 	    		(mark->level>0)?mgrd:lgrd, NULL);
 
 	    /* Polar axis tics are mirrored across the origin */
@@ -910,7 +918,7 @@ gen_tics(AXIS_INDEX axis, tic_callback callback)
 		int save_gridline = lgrd.l_type;
 		lgrd.l_type = LT_NODRAW;
 		(*callback) (axis, -internal,
-			(mark->level>0)?NULL:label,
+			(mark->level>0)?NULL:ticlabel,
 	    		(mark->level>0)?mgrd:lgrd, NULL);
 		lgrd.l_type = save_gridline;
 	    }
@@ -1405,8 +1413,8 @@ axis_position_zeroaxis(AXIS_INDEX axis)
     TBOOLEAN is_inside = FALSE;
     AXIS *this = axis_array + axis;
 
-    /* HBB 20020215: correctly treat reversed axes, too! */
-    /* EAM Sep 2005: Nothing wrong with 0 at extreme of the range */
+    /* NB: This is the only place that axis->term_zero is set. */
+    /*     So it is important to reach here before plotting.   */
     if ((this->min > 0.0 && this->max > 0.0)
 	|| this->log) {
 	this->term_zero = (this->max < this->min)
