@@ -1,5 +1,5 @@
-/* 
- * $Id: axis.h,v 1.36 2004/04/13 17:23:51 broeker Exp $
+/*
+ * $Id: axis.h,v 1.67.2.2 2013/09/27 00:25:39 sfeam Exp $
  *
  */
 
@@ -39,7 +39,7 @@
 #include "gp_types.h"		/* for TBOOLEAN */
 
 #include "gadgets.h"
-#include "parse.h"		/* for const_express() */
+#include "parse.h"		/* for const_*() */
 #include "tables.h"		/* for the axis name parse table */
 #include "term_api.h"		/* for lp_style_type */
 #include "util.h"		/* for int_error() */
@@ -48,7 +48,7 @@
 
 /* give some names to some array elements used in command.c and grap*.c
  * maybe one day the relevant items in setshow will also be stored
- * in arrays. 
+ * in arrays.
  *
  * Always keep the following conditions alive:
  * SECOND_X_AXIS = FIRST_X_AXIS + SECOND_AXES
@@ -56,28 +56,23 @@
  */
 typedef enum AXIS_INDEX {
 #define FIRST_AXES 0
-    FIRST_Z_AXIS, 
+    FIRST_Z_AXIS,
     FIRST_Y_AXIS,
     FIRST_X_AXIS,
-    T_AXIS,			/* fill gap */
+    COLOR_AXIS,			/* fill gap */
 #define SECOND_AXES 4
     SECOND_Z_AXIS,		/* not used, yet */
     SECOND_Y_AXIS,
     SECOND_X_AXIS,
-    R_AXIS,			/* never used ? */
-    U_AXIS,			/* ditto */
+    POLAR_AXIS,
+    T_AXIS,
+    U_AXIS,			/* never used? */
     V_AXIS			/* ditto */
-#ifdef PM3D
-    ,COLOR_AXIS
-#endif
+#define NO_AXIS 99
 } AXIS_INDEX;
 
-#ifdef PM3D
 # define AXIS_ARRAY_SIZE 11
-#else
-# define AXIS_ARRAY_SIZE 10
-#endif
-
+# define LAST_REAL_AXIS  POLAR_AXIS
 
 /* What kind of ticmarking is wanted? */
 typedef enum en_ticseries_type {
@@ -88,6 +83,12 @@ typedef enum en_ticseries_type {
     TIC_DAY      		/* print out day of week */
 } t_ticseries_type;
 
+typedef enum {
+    DT_NORMAL=0,		/* default; treat values as pure numeric */
+    DT_TIMEDATE,		/* old datatype */
+    DT_DMS			/* degrees minutes seconds */
+} td_type;
+
 /* Defines one ticmark for TIC_USER style.
  * If label==NULL, the value is printed with the usual format string.
  * else, it is used as the format string (note that it may be a constant
@@ -102,16 +103,19 @@ typedef struct ticmark {
 
 /* Tic-mark labelling definition; see set xtics */
 typedef struct ticdef {
-    t_ticseries_type type;	
+    t_ticseries_type type;
     char *font;
     struct t_colorspec textcolor;
-    union {
+    struct {
 	   struct ticmark *user;	/* for TIC_USER */
 	   struct {			/* for TIC_SERIES */
 		  double start, incr;
 		  double end;		/* ymax, if VERYLARGE */
 	   } series;
+	   TBOOLEAN mix;		/* TRUE to use both the above */
     } def;
+    struct position offset;
+    TBOOLEAN rangelimited;		/* Limit tics to data range */
 } t_ticdef;
 
 /* we want two auto modes for minitics - default where minitics are
@@ -129,7 +133,8 @@ enum en_minitics_status {
 
 /* Function pointer type for callback functions doing operations for a
  * single ticmark */
-typedef void (*tic_callback) __PROTO((AXIS_INDEX, double, char *, struct lp_style_type ));
+typedef void (*tic_callback) __PROTO((AXIS_INDEX, double, char *, struct lp_style_type,
+					struct ticmark *));
 
 /* Values to put in the axis_tics[] variables that decides where the
  * ticmarks should be drawn: not at all, on one or both plot borders,
@@ -161,7 +166,6 @@ typedef void (*tic_callback) __PROTO((AXIS_INDEX, double, char *, struct lp_styl
 #define GRID_MZ     (1<<7)
 #define GRID_MX2    (1<<8)
 #define GRID_MY2    (1<<9)
-/* GRID_{M}CB only for PM3D, but defined always for source code readability */
 #define GRID_CB     (1<<10)
 #define GRID_MCB    (1<<11)
 #endif /* 0 */
@@ -178,6 +182,13 @@ typedef enum e_autoscale {
     AUTOSCALE_FIXMAX = 1<<3
 } t_autoscale;
 
+typedef enum e_constraint {
+    CONSTRAINT_NONE  = 0,
+    CONSTRAINT_LOWER = 1<<0,
+    CONSTRAINT_UPPER = 1<<1,
+    CONSTRAINT_BOTH  = (1<<0 | 1<<1)
+} t_constraint;
+    
 
 /* FIXME 20000725: collect some of those various TBOOLEAN fields into
  * a larger int (or -- shudder -- a bitfield?) */
@@ -187,9 +198,9 @@ typedef struct axis {
     t_autoscale set_autoscale;	/* what does 'set' think autoscale to be? */
     int range_flags;		/* flag bits about autoscale/writeback: */
     /* write auto-ed ranges back to variables for autoscale */
-#define RANGE_WRITEBACK 1	
+#define RANGE_WRITEBACK 1
     /* allow auto and reversed ranges */
-#define RANGE_REVERSE   2	
+#define RANGE_REVERSE   2
     TBOOLEAN range_is_reverted;	/* range [high:low] silently reverted? */
     double min;			/* 'transient' axis extremal values */
     double max;
@@ -197,7 +208,15 @@ typedef struct axis {
     double set_max;
     double writeback_min;	/* ULIG's writeback implementation */
     double writeback_max;
+    double data_min;		/* Not necessarily the same as axis min */
+    double data_max;
 
+/* range constraints */
+    t_constraint min_constraint;
+    t_constraint max_constraint;
+    double min_lb, min_ub;     /* min lower- and upper-bound */
+    double max_lb, max_ub;     /* min lower- and upper-bound */
+    
 /* output-related quantities */
     int term_lower;		/* low and high end of the axis on output, */
     int term_upper;		/* ... (in terminal coordinates)*/
@@ -210,7 +229,7 @@ typedef struct axis {
     double log_base;		/* ln(base), for easier computations */
 
 /* time/date axis control */
-    TBOOLEAN is_timedata;	/* is this a time/date axis? */
+    td_type datatype;		/* DT_NORMAL | DT_TIMEDATE | DT_DMS */
     TBOOLEAN format_is_numeric;	/* format string looks like numeric??? */
     char timefmt[MAX_ID_LEN+1];	/* format string for input */
     char formatstring[MAX_ID_LEN+1];
@@ -224,34 +243,41 @@ typedef struct axis {
     TBOOLEAN gridminor;		/* Grid lines for minor tics? */
     int minitics;		/* minor tic mode (none/auto/user)? */
     double mtic_freq;		/* minitic stepsize */
+    double ticscale;		/* scale factor for tic marks (was (0..1])*/
+    double miniticscale;	/* and for minitics */
+    TBOOLEAN tic_in;		/* tics to be drawn inward?  */
 
 /* other miscellaneous fields */
-    label_struct label;		/* label string and position offsets */
+    text_label label;		/* label string and position offsets */
+    TBOOLEAN manual_justify;	/* override automatic justification */
     lp_style_type zeroaxis;	/* drawing style for zeroaxis, if any */
 } AXIS;
 
-#define DEFAULT_AXIS_TICDEF {TIC_COMPUTED, NULL, {TC_DEFAULT, 0, 0}, {NULL} }
-#ifdef PM3D
-# define DEFAULT_AXIS_ZEROAXIS {0, -3, 0, 1.0, 1.0, 0}
-#else
-# define DEFAULT_AXIS_ZEROAXIS {0, -3, 0, 1.0, 1.0}
-#endif
+#define DEFAULT_AXIS_TICDEF {TIC_COMPUTED, NULL, {TC_DEFAULT, 0, 0.0}, {NULL, {0.,0.,0.}, FALSE},  { character, character, character, 0., 0., 0. }, FALSE }
+#define DEFAULT_AXIS_ZEROAXIS {0, LT_NODRAW, 0, 0, 1.0, PTSZ_DEFAULT, FALSE, DEFAULT_COLORSPEC}
 
 #define DEFAULT_AXIS_STRUCT {						    \
 	AUTOSCALE_BOTH, AUTOSCALE_BOTH, /* auto, set_auto */		    \
 	0, FALSE,		/* range_flags, rev_range */		    \
-	-10.0, 10.0,		/* 3 pairs of min/max */		    \
+	-10.0, 10.0,		/* 3 pairs of min/max for axis itself */    \
 	-10.0, 10.0,							    \
 	-10.0, 10.0,							    \
-	0, 0, 0, 0,		/* terminal dependents */		    \
+	  0.0,  0.0,		/* and another min/max for the data */	    \
+	CONSTRAINT_NONE, CONSTRAINT_NONE,  /* min and max constraints */    \
+	0., 0., 0., 0.,         /* lower and upper bound for min and max */ \
+	0, 0,   		/* terminal lower and upper coords */	    \
+	0.,        		/* terminal scale */			    \
+	0,        		/* zero axis position */		    \
 	FALSE, 0.0, 0.0,	/* log, base, log(base) */		    \
-	0, 1,			/* is_timedata, format_numeric */	    \
-	DEF_FORMAT, TIMEFMT,	/* output format, timefmt */		    \
+	DT_NORMAL, TRUE,	/* datatype, format_numeric */	            \
+	TIMEFMT, DEF_FORMAT,	/* timefmt, output format */		    \
 	NO_TICS,		/* tic output positions (border, mirror) */ \
 	DEFAULT_AXIS_TICDEF,	/* tic series definition */		    \
 	0, FALSE, FALSE, 	/* tic_rotate, grid{major,minor} */	    \
-	MINI_DEFAULT, 10,	/* minitics, mtic_freq */		    \
+	MINI_DEFAULT, 10.,	/* minitics, mtic_freq */		    \
+        1.0, 0.5, TRUE,		/* ticscale, miniticscale, tic_in */	    \
 	EMPTY_LABELSTRUCT,	/* axis label */			    \
+	FALSE,			/* override automatic justification */	    \
 	DEFAULT_AXIS_ZEROAXIS	/* zeroaxis line style */		    \
 }
 
@@ -285,7 +311,7 @@ extern const struct ticdef default_axis_ticdef;
 #define TIMEFMT "%d/%m/%y,%H:%M"
 
 /* axis labels */
-extern const label_struct default_axis_label;
+extern const text_label default_axis_label;
 
 /* zeroaxis linetype (flag type==-3 if none wanted) */
 extern const lp_style_type default_axis_zeroaxis;
@@ -296,16 +322,15 @@ extern const struct lp_style_type default_grid_lp;
 /* grid layer: -1 default, 0 back, 1 front */
 extern int grid_layer;
 
+/* Whether or not to draw a separate polar axis in polar mode */
+extern TBOOLEAN raxis;
+
 /* global variables for communication with the tic callback functions */
 /* FIXME HBB 20010806: had better be collected into a struct that's
  * passed to the callback */
 extern int tic_start, tic_direction, tic_mirror;
 /* These are for passing on to write_multiline(): */
 extern int tic_text, rotate_tics, tic_hjust, tic_vjust;
-/* Some of them are controlled by 'set' commands: */
-extern double ticscale;		/* scale factor for tic marks (was (0..1])*/
-extern double miniticscale;	/* and for minitics */
-extern TBOOLEAN	tic_in;		/* tics to be drawn inward?  */
 /* The remaining ones are for grid drawing; controlled by 'set grid': */
 /* extern int grid_selection; --- comm'ed out, HBB 20010806 */
 extern struct lp_style_type grid_lp; /* linestyle for major grid lines */
@@ -315,6 +340,9 @@ extern double polar_grid_angle; /* angle step in polar grid in radians */
 /* Length of the longest tics label, set by widest_tic_callback(): */
 extern int widest_tic_strlen;
 
+/* flag to indicate that in-line axis ranges should be ignored */
+extern TBOOLEAN inside_zoom;
+
 /* axes being used by the current plot */
 extern AXIS_INDEX x_axis, y_axis, z_axis;
 /* macros to reduce code clutter caused by the array notation, mainly
@@ -322,9 +350,8 @@ extern AXIS_INDEX x_axis, y_axis, z_axis;
 #define X_AXIS axis_array[x_axis]
 #define Y_AXIS axis_array[y_axis]
 #define Z_AXIS axis_array[z_axis]
-#ifdef PM3D
+#define R_AXIS axis_array[POLAR_AXIS]
 #define CB_AXIS axis_array[COLOR_AXIS]
-#endif
 
 /* -------- macros using these variables: */
 
@@ -360,7 +387,7 @@ do {						\
 } while(0)
 
 /* HBB 20000430: New macros, logarithmize a value into a stored
- * coordinate*/ 
+ * coordinate*/
 #define AXIS_DO_LOG(axis,value) (log(value) / axis_array[axis].log_base)
 #define AXIS_UNDO_LOG(axis,value) exp((value) * axis_array[axis].log_base)
 
@@ -369,7 +396,7 @@ do {						\
     (axis_array[axis].log ? AXIS_DO_LOG(axis,value) : (value))
 #define AXIS_DE_LOG_VALUE(axis,coordinate)				  \
     (axis_array[axis].log ? AXIS_UNDO_LOG(axis,coordinate): (coordinate))
- 
+
 
 /* copy scalar data to arrays. The difference between 3D and 2D
  * versions is: dont know we have to support ranges [10:-10] - lets
@@ -398,6 +425,8 @@ do {									\
     } else {								\
 	this->log_base = this->log ? log(this->base) : 0;		\
     }									\
+    this->data_min = VERYLARGE;						\
+    this->data_max = -VERYLARGE;					\
 } while(0)
 
 #define AXIS_INIT2D(axis, infinite)					\
@@ -409,94 +438,66 @@ do {									\
 	? VERYLARGE : this->set_min;					\
     this->max = (infinite && (this->set_autoscale & AUTOSCALE_MAX))	\
 	? -VERYLARGE : this->set_max;					\
-    this->log_base = this->log ? log(this->base) : 0;			\
+    this->data_min = VERYLARGE;						\
+    this->data_max = -VERYLARGE;					\
 } while(0)
 
-/* handle reversed ranges */
-#define CHECK_REVERSE(axis) do {					\
+#ifdef VOLATILE_REFRESH
+#define AXIS_INIT2D_REFRESH(axis, infinite)				\
+do {									\
     AXIS *this = axis_array + axis;					\
 									\
-    if (((this->autoscale & AUTOSCALE_BOTH) == AUTOSCALE_NONE)		\
-	&& (this->max < this->min)) {					\
-	double temp = this->min;					\
-									\
-	this->min = this->max;						\
-	this->max = temp;						\
-	this->range_is_reverted = 1;					\
-    } else								\
-	this->range_is_reverted = (this->range_flags & RANGE_REVERSE);	\
+    this->autoscale = this->set_autoscale;				\
+    this->min = (infinite && (this->set_autoscale & AUTOSCALE_MIN))	\
+	? VERYLARGE*1e-3 : AXIS_LOG_VALUE(axis, this->set_min);		\
+    this->max = (infinite && (this->set_autoscale & AUTOSCALE_MAX))	\
+	? -VERYLARGE*1e-3 : AXIS_LOG_VALUE(axis, this->set_max);	\
+    this->log_base = this->log ? log(this->base) : 0;			\
 } while(0)
+/* why multiply by 1e-3: if an already VERYLARGE x2 and y2 ranges are
+   calculated after zoom-out by mouse, then they would become even larger
+*/
 
-/* HBB 20000725: new macro, built upon ULIG's SAVE_WRITEBACK(axis),
- * but easier to use. Code like this occured twice, in plot2d and
- * plot3d: */
-#define SAVE_WRITEBACK_ALL_AXES					\
-do {								\
-    AXIS_INDEX axis;						\
-								\
-    for (axis = 0; axis < AXIS_ARRAY_SIZE; axis++)		\
-	if(axis_array[axis].range_flags & RANGE_WRITEBACK) {	\
-	    set_writeback_min(axis);				\
-	    set_writeback_max(axis);				\
-	}							\
-} while(0)
-	    
-/* get optional [min:max] */
-#define PARSE_RANGE(axis)						   \
-do {									   \
-    if (equals(c_token, "[")) {						   \
-	c_token++;							   \
-	axis_array[axis].autoscale =					   \
-	    load_range(axis, &axis_array[axis].min, &axis_array[axis].max, \
-		       axis_array[axis].autoscale);			   \
-	if (!equals(c_token, "]"))					   \
-	    int_error(c_token, "']' expected");				   \
-	c_token++;							   \
-    }									   \
+#define AXIS_UPDATE2D_REFRESH(axis)					\
+do {									\
+    AXIS *this_axis = axis_array + axis;				\
+    if ((this_axis->set_autoscale & AUTOSCALE_MIN) == 0)		\
+	this_axis->min = AXIS_LOG_VALUE(axis, this_axis->set_min);	\
+    if ((this_axis->set_autoscale & AUTOSCALE_MAX) == 0)		\
+	this_axis->max = AXIS_LOG_VALUE(axis, this_axis->set_max);				\
 } while (0)
 
-/* HBB 20000430: new macro, like PARSE_RANGE, but for named ranges as
- * in 'plot [phi=3.5:7] sin(phi)' */
-#define PARSE_NAMED_RANGE(axis, dummy_token)				     \
-do {									     \
-    if (equals(c_token, "[")) {						     \
-	c_token++;							     \
-	if (isletter(c_token)) {					     \
-	    if (equals(c_token + 1, "=")) {				     \
-		dummy_token = c_token;					     \
-		c_token += 2;						     \
-	    }								     \
-		/* oops; probably an expression with a variable: act	     \
-		 * as if no variable name had been seen, by		     \
-		 * fallthrough */					     \
-	}								     \
-	axis_array[axis].autoscale = load_range(axis, &axis_array[axis].min, \
-				      &axis_array[axis].max,		     \
-				      axis_array[axis].autoscale);	     \
-	if (!equals(c_token, "]"))					     \
-	    int_error(c_token, "']' expected");				     \
-	c_token++;							     \
-    }				/* first '[' */				     \
-} while (0)
+#endif
+
+/* HBB NEW 20050316: macros to always access the actual minimum, even
+ * if 'set view map' or something else flipped things around behind
+ * our back */
+#define AXIS_ACTUAL_MIN(axis)				\
+    (axis_array[axis].range_flags & RANGE_REVERSE	\
+     ? axis_array[axis].max : axis_array[axis].min)
+
+#define AXIS_ACTUAL_MAX(axis)				\
+    (axis_array[axis].range_flags & RANGE_REVERSE	\
+     ? axis_array[axis].min : axis_array[axis].max)
+
 
 /* parse a position of the form
  *    [coords] x, [coords] y {,[coords] z}
- * where coords is one of first,second.graph,screen
- * if first or second, we need to take axis_is_timedata into account
+ * where coords is one of first,second.graph,screen,character
+ * if first or second, we need to take axis.datatype into account
  */
 #define GET_NUMBER_OR_TIME(store,axes,axis)				\
 do {									\
-    if (((axes) >= 0) && (axis_array[(axes)+(axis)].is_timedata)	\
-	&& isstring(c_token)) {						\
-	char ss[80];							\
+    if (((axes) >= 0) && (axis_array[(axes)+(axis)].datatype == DT_TIMEDATE)	\
+	&& isstringvalue(c_token)) {					\
 	struct tm tm;							\
-	quote_str(ss,c_token, 80);					\
-	++c_token;							\
-	if (gstrptime(ss,axis_array[axis].timefmt,&tm))			\
-	    (store) = (double) gtimegm(&tm);				\
+	double usec;							\
+	char *ss = try_to_get_string();					\
+	if (gstrptime(ss,axis_array[axis].timefmt,&tm,&usec))		\
+	    (store) = (double) gtimegm(&tm) + usec;			\
+	free(ss);							\
     } else {								\
-	struct value value;						\
-	(store) = real(const_express(&value));				\
+	(store) = real_expression();					\
     }									\
 } while(0)
 
@@ -514,15 +515,15 @@ do {							\
  * Do OUT_ACTION or UNDEF_ACTION as appropriate
  * adjust range provided type is INRANGE (ie dont adjust y if x is outrange
  * VALUE must not be same as STORE
+ * NOAUTOSCALE is per-plot property, whereas AUTOSCALE_XXX is per-axis.
  * Note: see the particular implementation for COLOR AXIS below.
  */
 
 #define STORE_WITH_LOG_AND_UPDATE_RANGE(STORE, VALUE, TYPE, AXIS,	  \
-				       OUT_ACTION, UNDEF_ACTION)	  \
+			        NOAUTOSCALE, OUT_ACTION, UNDEF_ACTION)	  \
 do {									  \
-    /* HBB 20000726: new check, to avoid crashes with axis index -1 */	  \
-    if (AXIS==-1)							  \
-	break;								  \
+    if (AXIS == NO_AXIS)                                                  \
+	break;                                                            \
     /* HBB 20040304: new check to avoid storing infinities and NaNs */	  \
     if (! (VALUE > -VERYLARGE && VALUE < VERYLARGE)) {			  \
 	TYPE = UNDEFINED;						  \
@@ -544,23 +545,51 @@ do {									  \
 	}								  \
     } else								  \
 	STORE = VALUE;							  \
+    if (NOAUTOSCALE)							  \
+	break;  /* this plot is not being used for autoscaling */	  \
     if (TYPE != INRANGE)						  \
 	break;  /* don't set y range if x is outrange, for example */	  \
     if ((int)AXIS < 0)							  \
 	break;	/* HBB 20000507: don't check range if not a coordinate */ \
+    if ( VALUE<axis_array[AXIS].data_min )				  \
+	axis_array[AXIS].data_min = VALUE;				  \
     if ( VALUE<axis_array[AXIS].min ) {					  \
-	if (axis_array[AXIS].autoscale & AUTOSCALE_MIN)			  \
-	    axis_array[AXIS].min = VALUE;				  \
-	else {								  \
+	if (axis_array[AXIS].autoscale & AUTOSCALE_MIN)	{		  \
+            if (axis_array[AXIS].min_constraint & CONSTRAINT_LOWER) {     \
+                if (axis_array[AXIS].min_lb <= VALUE) {                   \
+                    axis_array[AXIS].min = VALUE;                         \
+                } else {                                                  \
+                    axis_array[AXIS].min = axis_array[AXIS].min_lb;       \
+                    TYPE = OUTRANGE;                                       \
+                    OUT_ACTION;                                           \
+                    break;                                                \
+                }                                                         \
+            } else {                                                      \
+	        axis_array[AXIS].min = VALUE;				  \
+	    }								  \
+	} else {							  \
 	    TYPE = OUTRANGE;						  \
 	    OUT_ACTION;							  \
 	    break;							  \
 	}								  \
     }									  \
+    if ( VALUE>axis_array[AXIS].data_max )				  \
+	axis_array[AXIS].data_max = VALUE;				  \
     if ( VALUE>axis_array[AXIS].max ) {					  \
-	if (axis_array[AXIS].autoscale & AUTOSCALE_MAX)			  \
-	    axis_array[AXIS].max = VALUE;				  \
-	else {								  \
+	if (axis_array[AXIS].autoscale & AUTOSCALE_MAX)	{		  \
+	    if (axis_array[AXIS].max_constraint & CONSTRAINT_UPPER) {     \
+                if (axis_array[AXIS].max_ub >= VALUE) {                   \
+                    axis_array[AXIS].max = VALUE;                         \
+                } else {                                                  \
+                    axis_array[AXIS].max = axis_array[AXIS].max_ub;       \
+                    TYPE =OUTRANGE;                                       \
+                    OUT_ACTION;                                           \
+                    break;                                                \
+                }                                                         \
+            } else {                                                      \
+	        axis_array[AXIS].max = VALUE;                             \
+            }                                                     	  \
+	} else {							  \
 	    TYPE = OUTRANGE;						  \
 	    OUT_ACTION;							  \
 	}								  \
@@ -571,42 +600,24 @@ do {									  \
  * the type of the point (out-of-range color is plotted with the color
  * of the min or max color value).
  */
-#ifdef PM3D
 #define COLOR_STORE_WITH_LOG_AND_UPDATE_RANGE(STORE, VALUE, TYPE, AXIS,	  \
-				       OUT_ACTION, UNDEF_ACTION)	  \
+			       NOAUTOSCALE, OUT_ACTION, UNDEF_ACTION)	  \
 {									  \
-    int c_type_tmp = TYPE;						  \
+    coord_type c_type_tmp = TYPE;						  \
     STORE_WITH_LOG_AND_UPDATE_RANGE(STORE, VALUE, c_type_tmp, AXIS,	  \
-				       OUT_ACTION, UNDEF_ACTION);	  \
+			       NOAUTOSCALE, OUT_ACTION, UNDEF_ACTION);	  \
 }
-#endif
 
 /* Empty macro arguments triggered NeXT cpp bug       */
 /* #define NOOP (0) caused many warnings from gcc 3.2 */
 /* Now trying ((void)0) */
 #define NOOP ((void)0)
 
-/* HBB 20000506: new macro, initializes one variable to the same
- * value, for all axes. */
-#define INIT_AXIS_ARRAY(field, value)		\
-do {						\
-    int tmp;					\
-    for (tmp=0; tmp<AXIS_ARRAY_SIZE; tmp++)	\
-	axis_array[tmp].field=(value);		\
-} while(0)
-
 /* HBB 20000506: new macro to automatically build intializer lists
  * for arrays of AXIS_ARRAY_SIZE equal elements */
-#ifdef PM3D
 #define AXIS_ARRAY_INITIALIZER(value) {			\
     value, value, value, value, value,			\
 	value, value, value, value, value, value }
-#else
-#define AXIS_ARRAY_INITIALIZER(value) {		\
-    value, value, value, value, value,		\
-	value, value, value, value, value }
-#endif
-
 
 /* used by set.c */
 #define SET_DEFFORMAT(axis, flag_array)				\
@@ -621,10 +632,8 @@ do {						\
 /* (DFK) Watch for cancellation error near zero on axes labels */
 /* FIXME HBB 20000521: these seem not to be used much, anywhere... */
 #define CheckZero(x,tic) (fabs(x) < ((tic) * SIGNIF) ? 0.0 : (x))
-#define NearlyEqual(x,y,tic) (fabs((x)-(y)) < ((tic) * SIGNIF))
 
 
-	
 /* ------------ functions exported by axis.c */
 t_autoscale load_range __PROTO((AXIS_INDEX, double *, double *, t_autoscale));
 void axis_unlog_interval __PROTO((AXIS_INDEX, double *, double *, TBOOLEAN));
@@ -639,32 +648,31 @@ void axis_output_tics __PROTO((AXIS_INDEX, int *, AXIS_INDEX, tic_callback));
 void axis_set_graphical_range __PROTO((AXIS_INDEX, unsigned int lower, unsigned int upper));
 void axis_draw_2d_zeroaxis __PROTO((AXIS_INDEX, AXIS_INDEX));
 TBOOLEAN some_grid_selected __PROTO((void));
+void add_tic_user __PROTO((AXIS_INDEX, char *, double, int));
 
 double get_writeback_min __PROTO((AXIS_INDEX));
 double get_writeback_max __PROTO((AXIS_INDEX));
 void set_writeback_min __PROTO((AXIS_INDEX));
 void set_writeback_max __PROTO((AXIS_INDEX));
 
-/* set widest_tic_label: length of the longest tics label */
-void widest_tic_callback __PROTO((AXIS_INDEX, double place, char *text, struct lp_style_type grid));
+void save_writeback_all_axes __PROTO((void));
+void parse_range __PROTO((AXIS_INDEX axis));
+int  parse_named_range __PROTO((AXIS_INDEX axis, int token));
+void parse_skip_range __PROTO((void));
+void check_axis_reversed __PROTO((AXIS_INDEX axis));
 
-void get_position __PROTO((struct position * pos));
+/* set widest_tic_label: length of the longest tics label */
+void widest_tic_callback __PROTO((AXIS_INDEX, double place, char *text, 
+			struct lp_style_type grid, struct ticmark *));
+
+void get_position __PROTO((struct position *pos));
+void get_position_default __PROTO((struct position *pos, enum position_type default_type));
 
 /* ------------ autoscaling of the color axis */
-#ifdef PM3D
-
 #define NEED_PALETTE(plot) \
    (PM3DSURFACE == (plot)->plot_style \
     || PM3D_IMPLICIT == pm3d.implicit \
     || 1 == (plot)->lp_properties.use_palette)
 int set_cbminmax __PROTO((void));
 
-#else
-
-/* make this available regardless of the status of PM3D itself, for
- * ease of use */
-#define NEED_PALETTE(plot) FALSE
-
-#endif /* PM3D */
-
 #endif /* GNUPLOT_AXIS_H */
