@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: fit.c,v 1.69.2.1 2010/05/02 21:00:58 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: fit.c,v 1.78.2.7 2013/08/21 18:38:32 sfeam Exp $"); }
 #endif
 
 /*  NOTICE: Change of Copyright Status
@@ -76,17 +76,18 @@ static void Dblfn __PROTO(());
 #define Dblf5 Dblfn
 #define Dblf6 Dblfn
 
-#if defined(MSDOS) || defined(DOS386)	/* non-blocking IO stuff */
+#if defined(MSDOS) 	/* non-blocking IO stuff */
 # include <io.h>
-# ifndef _Windows		/* WIN16 does define MSDOS .... */
-#  include <conio.h>
-# endif
+# include <conio.h>
 # include <dos.h>
-#else /* !(MSDOS || DOS386) */
+#else /* !(MSDOS) */
 # ifndef VMS
 #  include <fcntl.h>
 # endif				/* !VMS */
-#endif /* !(MSDOS || DOS386) */
+#endif /* !(MSDOS) */
+#ifdef WIN32
+# include "win/winmain.h"
+#endif
 
 enum marq_res {
     OK, ML_ERROR, BETTER, WORSE
@@ -104,7 +105,7 @@ typedef enum marq_res marq_res_t;
 #define INITIAL_VALUE 1.0
 
 /* Relative change for derivatives */
-#define DELTA	    0.001
+#define DELTA       0.001
 
 #define MAX_DATA    2048
 #define MAX_PARAMS  32
@@ -113,7 +114,7 @@ typedef enum marq_res marq_res_t;
 #define LAMBDA_UP_FACTOR 10
 #define LAMBDA_DOWN_FACTOR 10
 
-#if defined(MSDOS) || defined(OS2) || defined(DOS386)
+#if defined(MSDOS) || defined(OS2)
 # define PLUSMINUS   "\xF1"	/* plusminus sign */
 #else
 # define PLUSMINUS   "+/-"
@@ -123,19 +124,19 @@ typedef enum marq_res marq_res_t;
 
 /* externally visible variables: */
 
-/* saved copy of last 'fit' command -- for output by "save" */
-char fitbuf[256];
-
 /* log-file for fit command */
 char *fitlogfile = NULL;
 TBOOLEAN fit_errorvariables = FALSE;
+TBOOLEAN fit_quiet = FALSE;
+TBOOLEAN ctrlc_flag = FALSE;
+char fitbuf[256]; /* for Eex and error_ex */
 
 /* private variables: */
 
 static int max_data;
 static int max_params;
 
-static double epsilon = 1e-5;	/* convergence limit */
+static double epsilon = DEF_FIT_LIMIT;	/* convergence limit */
 static int maxiter = 0;
 
 static char *fit_script = NULL;
@@ -166,7 +167,6 @@ static double *fit_x = 0;	/* all independent variable values,
 static double *fit_z = 0;	/* dependent data values */
 static double *err_data = 0;	/* standard deviations of dependent data */
 static double *a = 0;		/* array of fitting parameters */
-static TBOOLEAN ctrlc_flag = FALSE;
 static TBOOLEAN user_stop = FALSE;
 
 static struct udft_entry func;
@@ -181,7 +181,9 @@ static double startup_lambda = 0, lambda_down_factor = LAMBDA_DOWN_FACTOR, lambd
 			 internal Prototypes
 *****************************************************************/
 
+#if !defined(WIN32) || defined(WGP_CONSOLE)
 static RETSIGTYPE ctrlc_handle __PROTO((int an_int));
+#endif
 static void ctrlc_setup __PROTO((void));
 static marq_res_t marquardt __PROTO((double a[], double **alpha, double *chisq,
 				     double *lambda));
@@ -225,6 +227,7 @@ wri_to_fil_last_fit_cmd(FILE *fp)
     This is called when a SIGINT occurs during fit
 *****************************************************************/
 
+#if !defined(WIN32) || defined(WGP_CONSOLE)
 static RETSIGTYPE
 ctrlc_handle(int an_int)
 {
@@ -233,6 +236,7 @@ ctrlc_handle(int an_int)
     (void) signal(SIGINT, (sigfunc) ctrlc_handle);
     ctrlc_flag = TRUE;
 }
+#endif
 
 
 /*****************************************************************
@@ -250,7 +254,7 @@ ctrlc_setup()
  *
  *  I hope that other OSes do it better, if not... add #ifdefs :-(
  */
-#if (defined(__EMX__) || !defined(MSDOS) && !defined(DOS386))
+#if (defined(__EMX__) || !defined(MSDOS)) && (!defined(WIN32) || defined(WGP_CONSOLE))
     (void) signal(SIGINT, (sigfunc) ctrlc_handle);
 #endif
 }
@@ -259,7 +263,7 @@ ctrlc_setup()
 /*****************************************************************
     getch that handles also function keys etc.
 *****************************************************************/
-#if defined(MSDOS) || defined(DOS386)
+#if defined(MSDOS)
 
 /* HBB 980317: added a prototype... */
 int getchx __PROTO((void));
@@ -276,6 +280,7 @@ getchx()
 }
 #endif
 
+
 /*****************************************************************
     in case of fatal errors
 *****************************************************************/
@@ -287,8 +292,9 @@ error_ex()
     memcpy(fitbuf, "         ", 9);	/* start after GNUPLOT> */
     sp = strchr(fitbuf, NUL);
     while (*--sp == '\n');
-    strcpy(sp + 1, "\n\n");	/* terminate with exactly 2 newlines */
+    strcpy(sp + 1, "\n");
     fputs(fitbuf, STANDARD);
+    strcpy(sp + 2, "\n");	/* terminate with exactly 2 newlines */
     if (log_f) {
 	fprintf(log_f, "BREAK: %s", fitbuf);
 	(void) fclose(log_f);
@@ -301,10 +307,14 @@ error_ex()
     /* restore original SIGINT function */
     interrupt_setup();
 
-    bail_to_command_line();
+    /* exit via int_error() so that it can clean up state variables */
+    int_error(NO_CARET, "error during fit");
 }
 
+
 /* HBB 990829: removed the debug print routines */
+
+
 /*****************************************************************
     Marquardt's nonlinear least squares fit
 *****************************************************************/
@@ -361,6 +371,7 @@ marquardt(double a[], double **C, double *chisq, double *lambda)
 	free_matr(tmp_C);
 	return OK;
     }
+
     /* Givens calculates in-place, so make working copies of C and d */
 
     for (j = 0; j < num_data + num_params; j++)
@@ -377,11 +388,9 @@ marquardt(double a[], double **C, double *chisq, double *lambda)
     }
 
     /* FIXME: residues[] isn't used at all. Why? Should it be used? */
-
     Givens(tmp_C, tmp_d, da, residues, num_params + num_data, num_params, 1);
 
     /* check if trial did ameliorate sum of squares */
-
     for (j = 0; j < num_params; j++)
 	temp_a[j] = a[j] + da[j];
 
@@ -391,7 +400,8 @@ marquardt(double a[], double **C, double *chisq, double *lambda)
     }
     if (tmp_chisq < *chisq) {	/* Success, accept new solution */
 	if (*lambda > MIN_LAMBDA) {
-	    (void) putc('/', stderr);
+	    if (!fit_quiet)
+	        (void) putc('/', stderr);
 	    *lambda /= lambda_down_factor;
 	}
 	*chisq = tmp_chisq;
@@ -403,7 +413,8 @@ marquardt(double a[], double **C, double *chisq, double *lambda)
 	    a[j] = temp_a[j];
 	return BETTER;
     } else {			/* failure, increase lambda and return */
-	(void) putc('*', stderr);
+        if (!fit_quiet)
+	    (void) putc('*', stderr);
 	*lambda *= lambda_up_factor;
 	return WORSE;
     }
@@ -539,18 +550,21 @@ fit_interrupt()
 {
     while (TRUE) {
 	fputs("\n\n(S)top fit, (C)ontinue, (E)xecute FIT_SCRIPT:  ", STANDARD);
-	switch (getc(stdin)) {
+#ifdef WIN32
+	WinRaiseConsole();
+#endif
+	switch (getchar()) {
 
 	case EOF:
 	case 's':
 	case 'S':
-	    fputs("Stop.", STANDARD);
+	    fputs("Stop.\n", STANDARD);
 	    user_stop = TRUE;
 	    return FALSE;
 
 	case 'c':
 	case 'C':
-	    fputs("Continue.", STANDARD);
+	    fputs("Continue.\n", STANDARD);
 	    return TRUE;
 
 	case 'e':
@@ -566,8 +580,7 @@ fit_interrupt()
 		    (void) Gcomplex(&v, a[i], 0.0);
 		    setvar(par_name[i], v);
 		}
-		safe_strncpy(gp_input_line, tmp, gp_input_line_len);
-		(void) do_line();
+		do_string(tmp);
 	    }
 	}
     }
@@ -599,7 +612,8 @@ regress(double a[])
 	Eex("FIT: error occurred during fit");
     res = BETTER;
 
-    show_fit(iter, chisq, chisq, a, lambda, STANDARD);
+    if (!fit_quiet)
+        show_fit(iter, chisq, chisq, a, lambda, STANDARD);
     show_fit(iter, chisq, chisq, a, lambda, log_f);
 
     /* Reset flag describing fit result status */
@@ -611,6 +625,8 @@ regress(double a[])
 
     /* HBB 981118: initialize new variable 'user_break' */
     user_stop = FALSE;
+    /* FIXME: This really should not be necessary, but it is not properly initialised in wgnuplot otherwise. */
+    ctrlc_flag = FALSE;
 
     do {
 /*
@@ -626,7 +642,7 @@ regress(double a[])
  *  HBB: I think this can be enabled for DJGPP V2. SIGINT is actually
  *  handled there, AFAIK.
  */
-#if ((defined(MSDOS) || defined(DOS386)) && !defined(__EMX__))
+#if (defined(MSDOS) && !defined(__EMX__))
 	if (kbhit()) {
 	    do {
 		getchx();
@@ -634,6 +650,12 @@ regress(double a[])
 	    ctrlc_flag = TRUE;
 	}
 #endif
+#ifdef WIN32
+	/* This call makes the Windows GUI functional during fits.
+	   Pressing Ctrl-Break now finally has an effect. */
+	WinMessageLoop();
+#endif
+
 
 	if (ctrlc_flag) {
 	    show_fit(iter, chisq, last_chisq, a, lambda, STANDARD);
@@ -646,7 +668,8 @@ regress(double a[])
 	    last_chisq = chisq;
 	}
 	if ((res = marquardt(a, C, &chisq, &lambda)) == BETTER)
-	    show_fit(iter, chisq, last_chisq, a, lambda, STANDARD);
+	    if (!fit_quiet)
+	        show_fit(iter, chisq, last_chisq, a, lambda, STANDARD);
     } while ((res != ML_ERROR)
 	     && (lambda < MAX_LAMBDA)
 	     && ((maxiter == 0) || (iter <= maxiter))
@@ -667,6 +690,8 @@ regress(double a[])
 	Dblf2("\nMaximum iteration count (%d) reached. Fit stopped.\n", maxiter);
     } else if (user_stop) {
 	Dblf2("\nThe fit was stopped by the user after %d iterations.\n", iter);
+    } else if (lambda >= MAX_LAMBDA) {
+	Dblf2("\nThe maximum lambda = %e was exceeded. Fit stopped.\n", MAX_LAMBDA);
     } else {
 	Dblf2("\nAfter %d iterations the fit converged.\n", iter);
 	v = add_udv_by_name("FIT_CONVERGED");
@@ -755,9 +780,8 @@ regress(double a[])
 	}
 
 	/* scale parameter errors based on chisq */
-	chisq = sqrt(chisq / (num_data - num_params));
 	for (i = 0; i < num_params; i++)
-	    dpar[i] *= chisq;
+	    dpar[i] *= stdfit;
 
 	Dblf("Final set of parameters            Asymptotic Standard Error\n");
 	Dblf("=======================            ==========================\n\n");
@@ -1124,20 +1148,14 @@ update(char *pfile, char *npfile)
 static void
 backup_file(char *tofile, const char *fromfile)
 {
-#if defined (WIN32) || defined(MSDOS) || defined(VMS)
+#if defined(MSDOS) || defined(VMS)
     char *tmpn;
 #endif
 
-    /* win32 needs to have two attempts at the rename, since it may
-     * be running on win32s with msdos 8.3 names
-     */
-
 /* first attempt, for all o/s other than MSDOS */
 
 #ifndef MSDOS
-
     strcpy(tofile, fromfile);
-
 #ifdef VMS
     /* replace all dots with _, since we will be adding the only
      * dot allowed in VMS names
@@ -1145,17 +1163,15 @@ backup_file(char *tofile, const char *fromfile)
     while ((tmpn = strchr(tofile, '.')) != NULL)
 	*tmpn = '_';
 #endif /*VMS */
-
     strcat(tofile, BACKUP_SUFFIX);
-
     if (rename(fromfile, tofile) == 0)
 	return;			/* hurrah */
+    if (existfile(tofile))
+	Eex2("The backup file %s already exists and will not be overwritten.", tofile);
 #endif
 
-
-#if defined (WIN32) || defined(MSDOS)
-
-    /* first attempt for msdos. Second attempt for win32s */
+#ifdef MSDOS
+    /* first attempt for msdos. */
 
     /* Copy only the first 8 characters of the filename, to comply
      * with the restrictions of FAT filesystems. */
@@ -1168,13 +1184,13 @@ backup_file(char *tofile, const char *fromfile)
 
     if (rename(fromfile, tofile) == 0)
 	return;			/* success */
-
-#endif /* win32 || msdos */
+#endif /* MSDOS */
 
     /* get here => rename failed. */
     Eex3("Could not rename file %s to %s", fromfile, tofile);
 }
 
+
 /* A modified copy of save.c:save_range(), but this one reports
  * _current_ values, not the 'set' ones, by default */
 static void
@@ -1183,10 +1199,10 @@ log_axis_restriction(FILE *log_f, AXIS_INDEX axis, char *name)
     char s[80];
     AXIS *this_axis = axis_array + axis;
 
-    fprintf(log_f, "\t%s range restricted to [", name);
+    fprintf(log_f, "        %s range restricted to [", name);
     if (this_axis->autoscale & AUTOSCALE_MIN) {
 	putc('*', log_f);
-    } else if (this_axis->is_timedata) {
+    } else if (this_axis->datatype == DT_TIMEDATE) {
 	putc('"', log_f);
 	gstrftime(s, 80, this_axis->timefmt, this_axis->min);
 	fputs(s, log_f);
@@ -1198,7 +1214,7 @@ log_axis_restriction(FILE *log_f, AXIS_INDEX axis, char *name)
     fputs(" : ", log_f);
     if (this_axis->autoscale & AUTOSCALE_MAX) {
 	putc('*', log_f);
-    } else if (this_axis->is_timedata) {
+    } else if (this_axis->datatype == DT_TIMEDATE) {
 	putc('"', log_f);
 	gstrftime(s, 80, this_axis->timefmt, this_axis->max);
 	fputs(s, log_f);
@@ -1209,6 +1225,7 @@ log_axis_restriction(FILE *log_f, AXIS_INDEX axis, char *name)
     fputs("]\n", log_f);
 }
 
+
 /*****************************************************************
     Interface to the classic gnuplot-software
 *****************************************************************/
@@ -1234,6 +1251,7 @@ fit_command()
     time_t timer;
     int token1, token2, token3;
     char *tmp, *file_name;
+    AXIS saved_axis;
 
     c_token++;
 
@@ -1264,15 +1282,15 @@ fit_command()
       if (num_ranges > 5)
 	int_error(c_token, "only 6 range specs are permitted");
       i=var_order[num_ranges];
-      /* Store the Z axis dummy variable and range (if any) in the
-       * axis for the next independent variable.  Save the current
-       * values in an otherwise unused axis, so they can be restored
-       * later if there are fewer than 5 independent variables. */
-      axis_array[SECOND_Z_AXIS] = axis_array[i]; /* copy entire structure */
+      /* Store the Z axis dummy variable and range (if any) in the axis
+       * for the next independent variable.  Save the current values to be
+       * restored later if there are fewer than 5 independent variables.
+       */
+      saved_axis = axis_array[i];
       dummy_token[6] = dummy_token[i];
 
       dummy_token[num_ranges] = -1;
-      PARSE_NAMED_RANGE(i, dummy_token[num_ranges]); /* FIXME both should be i? */
+      dummy_token[num_ranges] = parse_named_range(i, dummy_token[num_ranges]);
       zrange_token = c_token;
       num_ranges++;
     }
@@ -1317,17 +1335,17 @@ fit_command()
     if (columns == 1)
 	int_error(c_token, "Need 2 to 7 using specs");
 
-    num_indep = (columns<3)?1:columns-2;
+    num_indep = (columns < 3) ? 1 : columns - 2;
 
     /* The following patch was made by Remko Scharroo, 25-Mar-1999
      * We need to check if one of the columns is time data, like
      * in plot2d and plot3d */
 
-    if (X_AXIS.is_timedata) {
+    if (X_AXIS.datatype == DT_TIMEDATE) {
 	if (columns < 2)
 	    int_error(c_token, "Need full using spec for x time data");
     }
-    if (Y_AXIS.is_timedata) {
+    if (Y_AXIS.datatype == DT_TIMEDATE) {
 	if (columns < 1)
 	    int_error(c_token, "Need using spec for y time data");
     }
@@ -1354,43 +1372,52 @@ fit_command()
     else if (num_ranges == num_indep+1 && num_indep < 5) {
       /* last range spec is for the Z axis */
       int i = var_order[num_ranges-1]; /* index for the last range spec */
-      
+
       Z_AXIS.autoscale = axis_array[i].autoscale;
       if (!(axis_array[i].autoscale & AUTOSCALE_MIN))
 	Z_AXIS.min = axis_array[i].min;
       if (!(axis_array[i].autoscale & AUTOSCALE_MAX))
 	Z_AXIS.max = axis_array[i].max;
-      
+
       /* restore former values */
-      axis_array[i] = axis_array[SECOND_Z_AXIS]; /* copy entire structure */
+      axis_array[i] = saved_axis;
       dummy_token[num_ranges-1] = dummy_token[6];
     }
 
     /* defer actually reading the data until we have parsed the rest
      * of the line */
-
     token3 = c_token;
 
     tmpd = getdvar(FITLIMIT);	/* get epsilon if given explicitly */
     if (tmpd < 1.0 && tmpd > 0.0)
 	epsilon = tmpd;
+    else
+	epsilon = DEF_FIT_LIMIT;
+    FPRINTF((STANDARD, "epsilon=%e\n", epsilon));
 
     /* HBB 970304: maxiter patch */
     maxiter = getivar(FITMAXITER);
+    if (maxiter < 0)
+	maxiter = 0;
+    FPRINTF((STANDARD, "maxiter=%i\n", maxiter));
 
     /* get startup value for lambda, if given */
     tmpd = getdvar(FITSTARTLAMBDA);
-
     if (tmpd > 0.0) {
 	startup_lambda = tmpd;
-	printf("Lambda Start value set: %g\n", startup_lambda);
+	Dblf2("lambda start value set: %g\n", startup_lambda);
+    } else {
+	startup_lambda = 0.0;
     }
+
     /* get lambda up/down factor, if given */
     tmpd = getdvar(FITLAMBDAFACTOR);
-
     if (tmpd > 0.0) {
 	lambda_up_factor = lambda_down_factor = tmpd;
-	printf("Lambda scaling factors reset:  %g\n", lambda_up_factor);
+	Dblf2("lambda scaling factors reset:  %g\n", lambda_up_factor);
+    } else {
+	lambda_down_factor = LAMBDA_DOWN_FACTOR;
+	lambda_up_factor = LAMBDA_UP_FACTOR;
     }
     free(fit_script);
     fit_script = NULL;
@@ -1400,12 +1427,10 @@ fit_command()
 
     {
 	char *logfile = getfitlogfile();
-
-	if (!log_f && !(log_f = fopen(logfile, "a")))
+	if ((logfile != NULL) && !log_f && !(log_f = fopen(logfile, "a")))
 	    Eex2("could not open log-file %s", logfile);
-
-	if (logfile)
-	    free(logfile);
+	FPRINTF((STANDARD, "log-file=%s\n", logfile));
+	free(logfile);
     }
 
     fputs("\n\n*******************************************************************************\n", log_f);
@@ -1418,21 +1443,21 @@ fit_command()
 	fprintf(log_f, "FIT:    data read from %s\n", line);
 	fprintf(log_f, "        format = ");
 	free(line);
-	for (i=0; i<num_indep && i<columns-1; i++)
-	  fprintf(log_f, "%s:", c_dummy_var[i]);
-	fprintf(log_f, (columns<=2) ? "z\n" : "z:s\n");
+	for (i = 0; (i < num_indep) && (i < columns - 1); i++)
+	    fprintf(log_f, "%s:", c_dummy_var[i]);
+	fprintf(log_f, (columns <= 2) ? "z\n" : "z:s\n");
     }
 
     /* report all range specs */
     j = FIRST_Z_AXIS;		/* check Z axis first */
-    for (i=0; i<=num_indep; i++) {
-      if ((axis_array[j].autoscale & AUTOSCALE_BOTH) != AUTOSCALE_BOTH)
-	log_axis_restriction(log_f, j, i ? c_dummy_var[i] : "z");
-      j=var_order[i];
+    for (i = 0; i <= num_indep; i++) {
+	if ((axis_array[j].autoscale & AUTOSCALE_BOTH) != AUTOSCALE_BOTH)
+	    log_axis_restriction(log_f, j, (i != 0) ? c_dummy_var[i-1] : "z");
+	j = var_order[i];
     }
 
     max_data = MAX_DATA;
-    fit_x = vec(max_data*num_indep); /* start with max. value */
+    fit_x = vec(max_data * num_indep); /* start with max. value */
     fit_z = vec(max_data);
     err_data = vec(max_data);
     num_data = 0;
@@ -1730,17 +1755,20 @@ Dblfn(const char *fmt, va_dcl)
 
     VA_START(args, fmt);
 # if defined(HAVE_VFPRINTF) || _LIBC
-    vfprintf(STANDARD, fmt, args);
+    if (!fit_quiet)
+        vfprintf(STANDARD, fmt, args);
     va_end(args);
     VA_START(args, fmt);
     vfprintf(log_f, fmt, args);
 # else
-    _doprnt(fmt, args, STANDARD);
+    if (!fit_quiet)
+        _doprnt(fmt, args, STANDARD);
     _doprnt(fmt, args, log_f);
 # endif
     va_end(args);
 #else
-    fprintf(STANDARD, fmt, a1, a2, a3, a4, a5, a6, a7, a8);
+    if (!fit_quiet)
+        fprintf(STANDARD, fmt, a1, a2, a3, a4, a5, a6, a7, a8);
     fprintf(log_f, fmt, a1, a2, a3, a4, a5, a6, a7, a8);
 #endif /* VA_START */
 }
