@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: mouse.c,v 1.133.2.2 2012/09/26 23:05:56 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: mouse.c,v 1.133.2.3 2012/10/27 02:24:37 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - mouse.c */
@@ -1829,6 +1829,13 @@ event_buttonrelease(struct gp_event_t *ge)
     load_mouse_variables(mouse_x, mouse_y, TRUE, b);
     UpdateStatusline();
 
+    /* In 2D mouse button 1 is available for "bind" commands */
+    if (!is_3d_plot && (b == 1)) {
+	ge->par1 = GP_Button1;
+	ge->par2 = 0;
+	event_keypress(ge, TRUE);
+    }
+
 #ifdef _Windows
     if (paused_for_mouse & PAUSE_CLICK) {
 	/* remove pause message box after 'pause mouse' */
