@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: term.c,v 1.225.2.9 2012/05/14 17:25:48 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: term.c,v 1.225.2.16 2013/08/23 09:50:36 mikulik Exp $"); }
 #endif
 
 /* GNUPLOT - term.c */
@@ -219,7 +219,6 @@ static double term_pointsize=1;
 
 static void term_suspend __PROTO((void));
 static void term_close_output __PROTO((void));
-static struct termentry *change_term __PROTO((const char *name, int length));
 
 static void null_linewidth __PROTO((double));
 static void do_point __PROTO((unsigned int x, unsigned int y, int number));
@@ -648,10 +647,12 @@ term_start_multiplot()
 
     term_start_plot();
 
+    /* FIXME: more options should be reset/initialized each time */
     mp_layout.auto_layout = FALSE;
     mp_layout.current_panel = 0;
+    mp_layout.title.noenhanced = FALSE;
 
-    /* Parse options (new in version 4.1 */
+    /* Parse options */
     while (!END_OF_COMMAND) {
 	char *s;
 
@@ -673,6 +674,18 @@ term_start_multiplot()
 	    continue;
 	}
 
+        if (almost_equals(c_token,"enh$anced")) {
+            mp_layout.title.noenhanced = FALSE;
+            c_token++;
+            continue;
+        }
+
+        if (almost_equals(c_token,"noenh$anced")) {
+            mp_layout.title.noenhanced = TRUE;
+            c_token++;
+            continue;
+        }
+
 	if (almost_equals(c_token, "lay$out")) {
 	    if (mp_layout.auto_layout)
 		int_error(c_token, "too many layout commands");
@@ -780,10 +793,17 @@ term_start_multiplot()
 
 	/* Calculate fractional height of title compared to entire page */
 	/* If it would fill the whole page, forget it! */
-	for (y=2; *p; p++)
+	for (y=1; *p; p++)
 	    if (*p == '\n')
 		y++;
+
+	/* Oct 2012 - v_char depends on the font used */
+	if (mp_layout.title.font && *mp_layout.title.font && term->set_font)
+	    term->set_font(mp_layout.title.font);
 	mp_layout.title_height = (double)(y * term->v_char) / (double)term->ymax;
+	if (mp_layout.title.font && *mp_layout.title.font && term->set_font)
+	    term->set_font("");
+
 	if (mp_layout.title_height > 0.9)
 	    mp_layout.title_height = 0.05;
     } else {
@@ -887,6 +907,7 @@ term_apply_lp_properties(struct lp_style_type *lp)
      *  this function by explicitly issuing additional '(*term)(...)'
      *  commands.
      */
+    t_colorspec colorspec = lp->pm3d_color;
     int lt = lp->l_type;
 
     if (lp->pointflag) {
@@ -912,9 +933,15 @@ term_apply_lp_properties(struct lp_style_type *lp)
 	(*term->linetype) (LT_BLACK);
     else
 	(*term->linetype) (lt);
+
     /* Possibly override the linetype color with a fancier colorspec */
-    if (lp->use_palette)
-	apply_pm3dcolor(&lp->pm3d_color, term);
+    if (!lp->use_palette) {
+	if ((term->flags & TERM_MONOCHROME) != 0)
+	    return;
+	colorspec.type = TC_LT;
+	colorspec.lt = lt;
+    }
+    apply_pm3dcolor(&colorspec, term);
 }
 
 
@@ -1616,7 +1643,7 @@ set_term()
  * returns NULL for unknown or ambiguous, otherwise is terminal
  * driver pointer
  */
-static struct termentry *
+struct termentry *
 change_term(const char *origname, int length)
 {
     int i;
@@ -1858,7 +1885,7 @@ test_term()
     struct termentry *t = term;
     const char *str;
     int x, y, xl, yl, i;
-    unsigned int xmax_t, ymax_t;
+    unsigned int xmax_t, ymax_t, x0, y0;
     char label[MAX_ID_LEN];
     int key_entry_height;
     int p_width;
@@ -1872,6 +1899,8 @@ test_term()
     screen_ok = FALSE;
     xmax_t = (unsigned int) (t->xmax * xsize);
     ymax_t = (unsigned int) (t->ymax * ysize);
+    x0 = (unsigned int) (xoffset * t->xmax);
+    y0 = (unsigned int) (yoffset * t->ymax);
 
     p_width = pointsize * t->h_tic;
     key_entry_height = pointsize * t->v_tic * 1.25;
@@ -1884,11 +1913,11 @@ test_term()
     /* border linetype */
     (*t->linewidth) (1.0);
     (*t->linetype) (LT_BLACK);
-    (*t->move) (0, 0);
-    (*t->vector) (xmax_t - 1, 0);
-    (*t->vector) (xmax_t - 1, ymax_t - 1);
-    (*t->vector) (0, ymax_t - 1);
-    (*t->vector) (0, 0);
+    (*t->move) (x0, y0);
+    (*t->vector) (x0 + xmax_t - 1, y0);
+    (*t->vector) (x0 + xmax_t - 1, y0 + ymax_t - 1);
+    (*t->vector) (x0, y0 + ymax_t - 1);
+    (*t->vector) (x0, y0);
     (*t->linetype)(0);
 
     /* Echo back the current terminal type */
@@ -1899,37 +1928,37 @@ test_term()
 	strcpy(tbuf,term->name);
 	strcat(tbuf,"  terminal test");
 	(void) (*t->justify_text) (LEFT);
-	(*t->put_text) (t->h_char * 2, ymax_t - t->v_char * 0.5, tbuf);
+	(*t->put_text) (x0 + t->h_char * 2, y0 + ymax_t - t->v_char * 0.5, tbuf);
     }
 
 #ifdef USE_MOUSE
     if (t->set_ruler) {
-	(*t->put_text) (t->h_char * 5, ymax_t - t->v_char * 3, "Mouse and hotkeys are supported, hit: h r m 6");
+	(*t->put_text) (x0 + t->h_char * 5, y0 + ymax_t - t->v_char * 3, "Mouse and hotkeys are supported, hit: h r m 6");
     }
 #endif
     (*t->linetype)(LT_BLACK);
     (*t->linetype) (LT_AXIS);
-    (*t->move) (xmax_t / 2, 0);
-    (*t->vector) (xmax_t / 2, ymax_t - 1);
-    (*t->move) (0, ymax_t / 2);
-    (*t->vector) (xmax_t - 1, ymax_t / 2);
+    (*t->move) (x0 + xmax_t / 2, y0);
+    (*t->vector) (x0 + xmax_t / 2, y0 + ymax_t - 1);
+    (*t->move) (x0, y0 + ymax_t / 2);
+    (*t->vector) (x0 + xmax_t - 1, y0 + ymax_t / 2);
     /* test width and height of characters */
     (*t->linetype) (3);
-    (*t->move) (xmax_t / 2 - t->h_char * 10, ymax_t / 2 + t->v_char / 2);
-    (*t->vector) (xmax_t / 2 + t->h_char * 10, ymax_t / 2 + t->v_char / 2);
-    (*t->vector) (xmax_t / 2 + t->h_char * 10, ymax_t / 2 - t->v_char / 2);
-    (*t->vector) (xmax_t / 2 - t->h_char * 10, ymax_t / 2 - t->v_char / 2);
-    (*t->vector) (xmax_t / 2 - t->h_char * 10, ymax_t / 2 + t->v_char / 2);
-    (*t->put_text) (xmax_t / 2 - t->h_char * 10, ymax_t / 2,
+    (*t->move) (x0 + xmax_t / 2 - t->h_char * 10, y0 + ymax_t / 2 + t->v_char / 2);
+    (*t->vector) (x0 + xmax_t / 2 + t->h_char * 10, y0 + ymax_t / 2 + t->v_char / 2);
+    (*t->vector) (x0 + xmax_t / 2 + t->h_char * 10, y0 + ymax_t / 2 - t->v_char / 2);
+    (*t->vector) (x0 + xmax_t / 2 - t->h_char * 10, y0 + ymax_t / 2 - t->v_char / 2);
+    (*t->vector) (x0 + xmax_t / 2 - t->h_char * 10, y0 + ymax_t / 2 + t->v_char / 2);
+    (*t->put_text) (x0 + xmax_t / 2 - t->h_char * 10, y0 + ymax_t / 2,
 		    "12345678901234567890");
-    (*t->put_text) (xmax_t / 2 - t->h_char * 10, ymax_t / 2 + t->v_char * 1.4,
+    (*t->put_text) (x0 + xmax_t / 2 - t->h_char * 10, y0 + ymax_t / 2 + t->v_char * 1.4,
 		    "test of character width:");
     (*t->linetype) (LT_BLACK);
 
     /* Test for enhanced text */
     if (t->flags & TERM_ENHANCED_TEXT) {
 	char *tmptext = gp_strdup("Enhanced text:   {x@_{0}^{n+1}}");
-	(*t->put_text) (xmax_t * 0.5, ymax_t * 0.40, tmptext);
+	(*t->put_text) (x0 + xmax_t * 0.5, y0 + ymax_t * 0.40, tmptext);
 	free(tmptext);
 	if (!already_in_enhanced_text_mode)
 	    do_string("set termopt noenh");
@@ -1937,42 +1966,42 @@ test_term()
 
     /* test justification */
     (void) (*t->justify_text) (LEFT);
-    (*t->put_text) (xmax_t / 2, ymax_t / 2 + t->v_char * 6, "left justified");
+    (*t->put_text) (x0 + xmax_t / 2, y0 + ymax_t / 2 + t->v_char * 6, "left justified");
     str = "centre+d text";
     if ((*t->justify_text) (CENTRE))
-	(*t->put_text) (xmax_t / 2,
-			ymax_t / 2 + t->v_char * 5, str);
+	(*t->put_text) (x0 + xmax_t / 2,
+			y0 + ymax_t / 2 + t->v_char * 5, str);
     else
-	(*t->put_text) (xmax_t / 2 - strlen(str) * t->h_char / 2,
-			ymax_t / 2 + t->v_char * 5, str);
+	(*t->put_text) (x0 + xmax_t / 2 - strlen(str) * t->h_char / 2,
+			y0 + ymax_t / 2 + t->v_char * 5, str);
     str = "right justified";
     if ((*t->justify_text) (RIGHT))
-	(*t->put_text) (xmax_t / 2,
-			ymax_t / 2 + t->v_char * 4, str);
+	(*t->put_text) (x0 + xmax_t / 2,
+			y0 + ymax_t / 2 + t->v_char * 4, str);
     else
-	(*t->put_text) (xmax_t / 2 - strlen(str) * t->h_char,
-			ymax_t / 2 + t->v_char * 4, str);
+	(*t->put_text) (x0 + xmax_t / 2 - strlen(str) * t->h_char,
+			y0 + ymax_t / 2 + t->v_char * 4, str);
     /* test text angle */
     (*t->linetype)(1);
     str = "rotated ce+ntred text";
     if ((*t->text_angle) (TEXT_VERTICAL)) {
 	if ((*t->justify_text) (CENTRE))
-	    (*t->put_text) (t->v_char,
-			    ymax_t / 2, str);
+	    (*t->put_text) (x0 + t->v_char,
+			    y0 + ymax_t / 2, str);
 	else
-	    (*t->put_text) (t->v_char,
-			    ymax_t / 2 - strlen(str) * t->h_char / 2, str);
+	    (*t->put_text) (x0 + t->v_char,
+			    y0 + ymax_t / 2 - strlen(str) * t->h_char / 2, str);
 	(*t->justify_text) (LEFT);
 	str = " rotated by +45 deg";
 	(*t->text_angle)(45);
-	(*t->put_text)(t->v_char * 3, ymax_t / 2, str);
+	(*t->put_text)(x0 + t->v_char * 3, y0 + ymax_t / 2, str);
 	(*t->justify_text) (LEFT);
 	str = " rotated by -45 deg";
 	(*t->text_angle)(-45);
-	(*t->put_text)(t->v_char * 2, ymax_t / 2, str);
+	(*t->put_text)(x0 + t->v_char * 2, y0 + ymax_t / 2, str);
     } else {
 	(void) (*t->justify_text) (LEFT);
-	(*t->put_text) (t->h_char * 2, ymax_t / 2 - t->v_char * 2, "can't rotate text");
+	(*t->put_text) (x0 + t->h_char * 2, y0 + ymax_t / 2 - t->v_char * 2, "can't rotate text");
     }
     (void) (*t->justify_text) (LEFT);
     (void) (*t->text_angle) (0);
@@ -1980,30 +2009,30 @@ test_term()
 
     /* test tic size */
     (*t->linetype)(4);
-    (*t->move) ((unsigned int) (xmax_t / 2 + t->h_tic * (1 + axis_array[FIRST_X_AXIS].ticscale)), (unsigned int) ymax_t - 1);
-    (*t->vector) ((unsigned int) (xmax_t / 2 + t->h_tic * (1 + axis_array[FIRST_X_AXIS].ticscale)),
-		  (unsigned int) (ymax_t - axis_array[FIRST_X_AXIS].ticscale * t->v_tic));
-    (*t->move) ((unsigned int) (xmax_t / 2), (unsigned int) (ymax_t - t->v_tic * (1 + axis_array[FIRST_X_AXIS].ticscale)));
-    (*t->vector) ((unsigned int) (xmax_t / 2 + axis_array[FIRST_X_AXIS].ticscale * t->h_tic),
-		  (unsigned int) (ymax_t - t->v_tic * (1 + axis_array[FIRST_X_AXIS].ticscale)));
+    (*t->move) ((unsigned int) (x0 + xmax_t / 2 + t->h_tic * (1 + axis_array[FIRST_X_AXIS].ticscale)), y0 + (unsigned int) ymax_t - 1);
+    (*t->vector) ((unsigned int) (x0 + xmax_t / 2 + t->h_tic * (1 + axis_array[FIRST_X_AXIS].ticscale)),
+		  (unsigned int) (y0 + ymax_t - axis_array[FIRST_X_AXIS].ticscale * t->v_tic));
+    (*t->move) ((unsigned int) (x0 + xmax_t / 2), y0 + (unsigned int) (ymax_t - t->v_tic * (1 + axis_array[FIRST_X_AXIS].ticscale)));
+    (*t->vector) ((unsigned int) (x0 + xmax_t / 2 + axis_array[FIRST_X_AXIS].ticscale * t->h_tic),
+		  (unsigned int) (y0 + ymax_t - t->v_tic * (1 + axis_array[FIRST_X_AXIS].ticscale)));
     /* HBB 19990530: changed this to use right-justification, if possible... */
     str = "show ticscale";
     if ((*t->justify_text) (RIGHT))
-	(*t->put_text) ((unsigned int) (xmax_t / 2 - 1* t->h_char),
-			(unsigned int) (ymax_t - (t->v_tic * 2 + t->v_char / 2)),
+	(*t->put_text) (x0 + (unsigned int) (xmax_t / 2 - 1* t->h_char),
+			y0 + (unsigned int) (ymax_t - (t->v_tic * 2 + t->v_char / 2)),
 		    str);
     else
-	(*t->put_text) ((unsigned int) (xmax_t / 2 - (strlen(str)+1)     * t->h_char),
-			(unsigned int) (ymax_t - (t->v_tic * 2 + t->v_char / 2)),
+	(*t->put_text) (x0 + (unsigned int) (xmax_t / 2 - (strlen(str)+1) * t->h_char),
+			y0 + (unsigned int) (ymax_t - (t->v_tic * 2 + t->v_char / 2)),
 			str);
     (void) (*t->justify_text) (LEFT);
     (*t->linetype)(LT_BLACK);
 
     /* test line and point types */
-    x = xmax_t - t->h_char * 6 - p_width;
-    y = ymax_t - key_entry_height;
+    x = x0 + xmax_t - t->h_char * 6 - p_width;
+    y = y0 + ymax_t - key_entry_height;
     (*t->pointsize) (pointsize);
-    for (i = -2; y > key_entry_height; i++) {
+    for (i = -2; y > y0 + key_entry_height; i++) {
 	struct lp_style_type ls = DEFAULT_LP_STYLE_TYPE;
 	ls.l_width = 1;
 	load_linetype(&ls,i+1);
@@ -2024,8 +2053,8 @@ test_term()
     /* test some arrows */
     (*t->linewidth) (1.0);
     (*t->linetype) (0);
-    x = xmax_t * .375;
-    y = ymax_t * .250;
+    x = x0 + xmax_t * .375;
+    y = y0 + ymax_t * .250;
     xl = t->h_tic * 7;
     yl = t->v_tic * 7;
     i = curr_arrow_headfilled;
@@ -2049,8 +2078,8 @@ test_term()
     (void) (*t->justify_text) (LEFT);
     xl = xmax_t / 10;
     yl = ymax_t / 25;
-    x = xmax_t * .075;
-    y = yl;
+    x = x0 + xmax_t * .075;
+    y = y0 + yl;
 
     for (i=1; i<7; i++) {
 	(*t->linewidth) ((float)(i)); (*t->linetype)(LT_BLACK);
@@ -2062,14 +2091,14 @@ test_term()
     (*t->put_text) (x, y, "linewidth");
 
     /* test fill patterns */
-    x = xmax_t * 0.5;
-    y = 0;
+    x = x0 + xmax_t * 0.5;
+    y = y0;
     xl = xmax_t / 40;
     yl = ymax_t / 8;
     (*t->linewidth) ((float)(1));
     (*t->linetype)(LT_BLACK);
     (*t->justify_text) (CENTRE);
-    (*t->put_text)(x+xl*7, yl+t->v_char*1.5, "pattern fill");
+    (*t->put_text)(x+xl*7, y + yl+t->v_char*1.5, "pattern fill");
     for (i=0; i<10; i++) {
 	int style = ((i<<4) + FS_PATTERN);
 	if (t->fillbox)
@@ -2085,8 +2114,8 @@ test_term()
     }
 
     {
-	int cen_x = (int)(0.70 * xmax_t);
-	int cen_y = (int)(0.83 * ymax_t);
+	int cen_x = x0 + (int)(0.70 * xmax_t);
+	int cen_y = y0 + (int)(0.83 * ymax_t);
 	int radius = xmax_t / 20;
 
 	/* test pm3d -- filled_polygon(), but not set_color() */
@@ -2562,6 +2591,8 @@ enhanced_recursion(
 	    p = enhanced_recursion(++p, FALSE, fontname, fontsize, base,
 			      widthflag, showflag, 1);
 	    (term->enhanced_flush)();
+	    if (!*p)
+	        break;
 	    p = enhanced_recursion(++p, FALSE, fontname, fontsize, base,
 			      FALSE, showflag, 2);
 
