@@ -1,5 +1,5 @@
 /*
- * $Id: util3d.h,v 1.15 2008/07/08 04:56:45 sfeam Exp $
+ * $Id: util3d.h,v 1.18.2.1 2012/01/11 23:42:23 sfeam Exp $
  */
 
 /* GNUPLOT - util3d.h */
@@ -43,6 +43,7 @@
  * known */
 
 #include "graph3d.h"
+#include "graphics.h"	/* only needed for apply_head_properties() */
 
 /* All the necessary information about one vertex. */
 typedef struct vertex {
@@ -74,8 +75,8 @@ typedef vertex GPHUGE * p_vertex;
 
 /* Prototypes of functions exported by "util3d.c" */
 
-void edge3d_intersect __PROTO((struct coordinate GPHUGE *, int, double *, double *, double *));
-TBOOLEAN two_edge3d_intersect __PROTO((struct coordinate GPHUGE *, int, double *, double *, double *));
+void edge3d_intersect __PROTO((struct coordinate GPHUGE *, int, int, double *, double *, double *));
+TBOOLEAN two_edge3d_intersect __PROTO((struct coordinate GPHUGE *, int, int, double *, double *, double *));
 void mat_scale __PROTO((double sx, double sy, double sz, double mat[4][4]));
 void mat_rot_x __PROTO((double teta, double mat[4][4]));
 void mat_rot_z __PROTO((double teta, double mat[4][4]));
@@ -84,8 +85,10 @@ void map3d_xyz __PROTO((double x, double y, double z, p_vertex out));
 void map3d_xy __PROTO((double x, double y, double z, int *xt, int *yt));
 void map3d_xy_double __PROTO((double x, double y, double z, double *xt, double *yt));
 void draw3d_line __PROTO((p_vertex, p_vertex, struct lp_style_type *));
-void draw3d_line_unconditional __PROTO((p_vertex, p_vertex, struct lp_style_type *, int));
+void draw3d_line_unconditional __PROTO((p_vertex, p_vertex, struct lp_style_type *, t_colorspec));
 void draw3d_point __PROTO((p_vertex v, struct lp_style_type *lp));
+void apply_3dhead_properties __PROTO((struct arrow_style_type *arrow_properties));
+
 
 /* HBB NEW 20031218: 3D polyline support */
 void polyline3d_start __PROTO((p_vertex v1));
