@@ -1,5 +1,5 @@
 /*
- * $Id: gp_cairo.c,v 1.48.2.4 2010/08/26 18:18:27 sfeam Exp $
+ * $Id: gp_cairo.c,v 1.60.2.5 2013/05/24 19:42:05 sfeam Exp $
  */
 
 /* GNUPLOT - gp_cairo.c */
@@ -140,12 +140,17 @@ static rgb_color gp_cairo_colorlist[12] = {
 };
 
 /* correspondance between gnuplot linetypes and terminal colors */
+void gp_cairo_set_background( rgb_color background )
+{
+	gp_cairo_colorlist[0] = background;
+}
+
 rgb_color gp_cairo_linetype2color( int linetype )
 {
 	if (linetype<=LT_NODRAW)
-		linetype = LT_NODRAW; /* background color*/
-
-	return gp_cairo_colorlist[ linetype%9 +3 ];
+		return gp_cairo_colorlist[ 0 ];
+	else
+		return gp_cairo_colorlist[ linetype%9 +3 ];
 }
 
 /* initialize all fields of the plot structure */
@@ -163,6 +168,7 @@ void gp_cairo_initialize_plot(plot_struct *plot)
 	plot->dashlength = 1.0;
 	plot->text_angle = 0.0;
 	plot->color.r = 0.0; plot->color.g = 0.0; plot->color.b = 0.0;
+	plot->background.r = 1.0; plot->background.g = 1.0; plot->background.b = 1.0;
 
 	plot->opened_path = FALSE;
 
@@ -289,8 +295,14 @@ void gp_cairo_set_font(plot_struct *plot, const char *name, int fontsize)
 
 	/* Split out Bold and Italic attributes from font name */
 	fname = strdup(name);
-	for (c=fname; *c; c++)
-	    if (*c == '-') *c = ' ';
+	for (c=fname; *c; c++) {
+	    if (*c == '\\') {
+		char *d = c;
+		do { *d = *(d+1); } while (*d++);
+	    } else {
+		if (*c == '-') *c = ' ';
+	    }
+	}
 	if ((c = strstr(fname, " Bold"))) {
 	    do { *c = *(c+5); } while (*c++);
 	    plot->fontweight = PANGO_WEIGHT_BOLD;
@@ -304,7 +316,6 @@ void gp_cairo_set_font(plot_struct *plot, const char *name, int fontsize)
 
 	strncpy( plot->fontname, fname, sizeof(plot->fontname) );
 	plot->fontsize = fontsize;
-
 	free(fname);
 }
 
@@ -329,38 +340,37 @@ void gp_cairo_set_textangle(plot_struct *plot, double angle)
 	plot->text_angle =angle;
 }
 
-/* By default, Cairo uses an antialiasing algorithm which
- * will let a seam between polygons which
- * share common edges.
+/* By default, Cairo uses an antialiasing algorithm which may
+ * leave a seam between polygons which share a common edge.
  * Several solutions allow to workaround this behaviour :
  * - don't antialias the polygons
- * Problem : aliased lines are ugly
+ *   Problem : aliased lines are ugly
  * - stroke on each edge
- * Problem : stroking is a very time-consuming operation
+ *   Problem : stroking is a very time-consuming operation
  * - draw without antialiasing to a separate context of a bigger size
- * Problem : not really in the spirit of the rest of the drawing.
- * - dilatate the polygon so that they overlap slightly
- * It is really more time-consuming that it may seem. It implies dealing
- * with each corner, find on which direction to move it (making
- * the difference between the inside and the outside of the polygon).
+ *   Problem : not really in the spirit of the rest of the drawing.
+ * - enlarge the polygons so that they overlap slightly
+ *   Problem : It is really more time-consuming that it may seem.
+ *   It implies inspecting each corner to find which direction to move it
+ *   (making the difference between the inside and the outside of the polygon).
  * - using CAIRO_OPERATOR_SATURATE
- * Problem : for each set of polygons, we have to draw front-to-back
- * on a separate context and then copy back to this one.
- * Time-consuming but probably less than stroking all the edges.
+ *   Problem : for each set of polygons, we have to draw front-to-back
+ *   on a separate context and then copy back to this one.
+ *   Time-consuming but probably less than stroking all the edges.
  *
  * The last solution is implemented if plot->polygons_saturate is set to TRUE
- * Otherwise the default (antialiasing but seams) is used.
+ * Otherwise the default (antialiasing but may have seams) is used.
  */
 
 void gp_cairo_draw_polygon(plot_struct *plot, int n, gpiPoint *corners)
 {
+	/* begin by stroking any open path */
+	gp_cairo_stroke(plot);
+	
 	if (plot->polygons_saturate) {
 		int i;
 		path_item *path;
 	
-		/* begin by stroking any open path */
-		gp_cairo_stroke(plot);
-	
 		path = (path_item*) gp_alloc(sizeof(path_item), "gp_cairo : polygon path");
 	
 		path->n = n;
@@ -675,7 +685,7 @@ gchar * gp_cairo_convert(plot_struct *plot, const char* string)
 		if (error != NULL) {
 			fprintf(stderr, "Unable to convert \"%s\": the sequence is invalid "\
 				"in the current charset (%s), %d bytes read out of %d\n",
-				string, charset, bytes_read, strlen(string));
+				string, charset, (int)bytes_read, (int)strlen(string));
 			string_utf8 = g_convert(string, bytes_read, "UTF-8", charset, NULL, NULL, NULL);
 			g_error_free (error);
 		} else
@@ -692,8 +702,10 @@ gchar * gp_cairo_convert(plot_struct *plot, const char* string)
  * the cairo/win32 backend for font rendering.  It has the effect of
  * testing for libfreetype support, and using that instead if possible.
  * Suggested by cairo developer Behdad Esfahbod. 
+ * Allin Cottrell suggests that this not necessary anymore for newer
+ * versions of cairo.
  */
-#ifdef WIN32
+#if defined(WIN32) && (CAIRO_VERSION_MAJOR < 2) && (CAIRO_VERSION_MINOR < 10)
 PangoLayout *
 gp_cairo_create_layout (cairo_t *cr)
 {
@@ -754,7 +766,7 @@ void gp_cairo_draw_text(plot_struct *plot, int x1, int y1, const char* string)
 	if (!strcmp(plot->fontname,"Symbol")) {
 		FPRINTF((stderr,"Parsing a Symbol string\n"));
 		string_utf8 = gp_cairo_convert_symbol_to_unicode(plot, string);
-		strncpy(plot->fontname, "Sans", sizeof(plot->fontname));
+		strncpy(plot->fontname, gp_cairo_default_font(), sizeof(plot->fontname));
 		symbol_font_parsed = TRUE;
 	} else
 #endif /*MAP_SYMBOL*/
@@ -786,10 +798,15 @@ void gp_cairo_draw_text(plot_struct *plot, int x1, int y1, const char* string)
 	
 	pango_layout_get_extents(layout, &ink_rect, &logical_rect);
 
-	/* EAM Mar 2009 - Adjusting the vertical position for every character fragment */
-	/* leads to uneven baselines.  Better to adjust to the "average" character height */
-	/* vert_just = ((double)ink_rect.height/2 +(double)ink_rect.y) / PANGO_SCALE; */
-	vert_just = avg_vchar/2;
+	/* EAM Mar 2009 - Adjusting the vertical position for every character fragment	*/
+	/* leads to uneven baselines. Better to adjust to the "average" character height */
+	/* EAM Dec 2012 - The problem is that avg_vchar is not kept in sync with the	*/
+	/* font size.  It is changed when the set_font command is received, not when	*/
+	/* it is executed in the display list. Try basing off plot->fontsize instead. 	*/
+
+	/* vert_just = ((double)ink_rect.height/2 +(double)ink_rect.y) / PANGO_SCALE;	*/
+	/* vert_just = avg_vchar/2;							*/
+	vert_just = 0.8 * (float)(plot->fontsize * plot->oversampling_scale);
 
 	x = (double) x1;
 	y = (double) y1;
@@ -902,9 +919,11 @@ void gp_cairo_draw_point(plot_struct *plot, int x1, int y1, int style)
 	cairo_set_line_width(plot->cr, plot->linewidth*plot->oversampling_scale);
 	cairo_set_source_rgb(plot->cr, plot->color.r, plot->color.g, plot->color.b);
 
-	/* always draw a dot. Nothing more is needed for style=-1 */
-	cairo_arc (plot->cr, x, y, 0.5*plot->oversampling_scale, 0, 2*M_PI);
-	cairo_fill (plot->cr);
+	/* Dot	FIXME: because this is drawn as a filled circle, it's quite slow */
+	if (style < 0) {
+		cairo_arc (plot->cr, x, y, 0.5*plot->oversampling_scale, 0, 2*M_PI);
+		cairo_fill (plot->cr);
+	}
 
 	switch (style%13) {
 	case 0: /* plus */
@@ -1032,10 +1051,10 @@ void gp_cairo_draw_fillbox(plot_struct *plot, int x, int y, int width, int heigh
 	gp_cairo_fill( plot, fillstyle, fillpar);
 
 	cairo_move_to(plot->cr, x, y);
-	cairo_rel_line_to(plot->cr,0, -height);
-	cairo_rel_line_to(plot->cr, width-1, 0);
+	cairo_rel_line_to(plot->cr, 0, -height);
+	cairo_rel_line_to(plot->cr, width, 0);
 	cairo_rel_line_to(plot->cr, 0, height);
-	cairo_rel_line_to(plot->cr, -width+1, 0);
+	cairo_rel_line_to(plot->cr, -width, 0);
 	cairo_close_path(plot->cr);
 	cairo_fill(plot->cr);
 }
@@ -1130,7 +1149,7 @@ void gp_cairo_add_shape( PangoRectangle rect,int position)
 
 	FPRINTF((stderr, "adding blank custom shape\n"));
 
-	strncat(gp_cairo_utf8, " ", sizeof(gp_cairo_utf8)-strlen(gp_cairo_utf8));
+	strncat(gp_cairo_utf8, " ", sizeof(gp_cairo_utf8)-strlen(gp_cairo_utf8)-1);
 	p_attr_shape = pango_attr_shape_new (&rect,&rect);
 	p_attr_shape->start_index = position;
 	p_attr_shape->end_index = position+1;
@@ -1198,7 +1217,7 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 				sizeof(gp_cairo_enhanced_font));
 		} else {
 			strncpy(gp_cairo_enhanced_font,
-				"Sans", sizeof(gp_cairo_enhanced_font));
+				gp_cairo_default_font(), sizeof(gp_cairo_enhanced_font));
 		}
 		symbol_font_parsed = TRUE;
 	} else
@@ -1243,7 +1262,6 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 		 * the saved attributes list, get extents */
 		underprinted_layout = gp_cairo_create_layout (plot->cr);
 		pango_layout_set_text (underprinted_layout, gp_cairo_underprinted_utf8, -1);
-		// EAM DEBUG
 		if (!gp_cairo_enhanced_underprinted_AttrList)
 			fprintf(stderr,"uninitialized gp_cairo_enhanced_underprinted_AttrList!\n");
 		else
@@ -1282,7 +1300,7 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 	}
 
 	if (gp_cairo_enhanced_showflag) {
-		strncat(gp_cairo_utf8, enhanced_text_utf8, sizeof(gp_cairo_utf8)-strlen(gp_cairo_utf8));
+		strncat(gp_cairo_utf8, enhanced_text_utf8, sizeof(gp_cairo_utf8)-strlen(gp_cairo_utf8)-1);
 		end = strlen(gp_cairo_utf8);
 
 		/* add text attributes to the main list */
@@ -1363,7 +1381,7 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 
 	if (gp_cairo_enhanced_save) /* we aim at restoring position later */ {
 		save_start = strlen( gp_cairo_save_utf8);
-		strncat(gp_cairo_save_utf8, enhanced_text_utf8, sizeof(gp_cairo_utf8)-strlen(gp_cairo_utf8));
+		strncat(gp_cairo_save_utf8, enhanced_text_utf8, sizeof(gp_cairo_utf8)-strlen(gp_cairo_utf8)-1);
 		save_end = strlen( gp_cairo_save_utf8);
 
 		/* add text attributes to the save list */
@@ -1374,7 +1392,7 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 		underprinted_start = strlen(gp_cairo_underprinted_utf8);
 		strncat(gp_cairo_underprinted_utf8,
 			enhanced_text_utf8,
-			sizeof(gp_cairo_underprinted_utf8)-strlen(gp_cairo_underprinted_utf8));
+			sizeof(gp_cairo_underprinted_utf8)-strlen(gp_cairo_underprinted_utf8)-1);
 		underprinted_end = strlen(gp_cairo_underprinted_utf8);
 
 		if (gp_cairo_enhanced_underprinted_AttrList)
@@ -1557,7 +1575,7 @@ void gp_cairo_fill(plot_struct *plot, int fillstyle, int fillpar)
 		FPRINTF((stderr,"pattern fillpar = %d %lf %lf %lf\n",fillpar, plot->color.r, plot->color.g, plot->color.b));
 		return;
 	case FS_EMPTY: /* fill with background plot->color */
-		cairo_set_source_rgb(plot->cr, 1, 1, 1);
+		cairo_set_source_rgb(plot->cr, plot->background.r, plot->background.g, plot->background.b);
 		FPRINTF((stderr,"empty\n"));
 		return;
 	default:
@@ -1596,7 +1614,10 @@ void gp_cairo_fill_pattern(plot_struct *plot, int fillstyle, int fillpar)
 	    cairo_set_source_rgb( pattern_cr, 1.0, 1.0, 1.0);
 
 	cairo_paint(pattern_cr);
-	cairo_set_line_width(pattern_cr, PATTERN_SIZE/50.);
+	if (!strcmp(term->name,"pdfcairo")) /* Work-around for poor scaling in cairo <= 1.10 */
+	    cairo_set_line_width(pattern_cr, PATTERN_SIZE/150.);
+	else
+	    cairo_set_line_width(pattern_cr, PATTERN_SIZE/50.);
 	cairo_set_line_cap  (pattern_cr, CAIRO_LINE_CAP_BUTT);
 	cairo_set_source_rgb(pattern_cr, plot->color.r, plot->color.g, plot->color.b);
 
@@ -1706,6 +1727,9 @@ void gp_cairo_set_termvar(plot_struct *plot, unsigned int *v_char,
 	if (h_char)
 		*h_char = tmp_h_char;
 
+/* FIXME!!! So far, so good. But now we have a problem. This routine	*/
+/* is called synchronously with the set_font command, but avg_vchar is	*/
+/* needed asynchronously during execution of the display list. 		*/
 	avg_vchar = tmp_v_char;
 }
 
@@ -1716,7 +1740,7 @@ void gp_cairo_solid_background(plot_struct *plot)
 			cairo_status_to_string (cairo_status (plot->cr)));
 		exit(0);
 	}
-	cairo_set_source_rgb(plot->cr, 1.0, 1.0, 1.0);
+	cairo_set_source_rgb(plot->cr, plot->background.r, plot->background.g, plot->background.b);
 	cairo_paint(plot->cr);
 }
 
@@ -1746,6 +1770,19 @@ const char* gp_cairo_enhanced_get_fontname(plot_struct *plot)
 		return gp_cairo_enhanced_font;
 }
 
+/* BM: New function to determine the default font.
+ * On Windows, the "Sans" alias normally is equivalent to 
+ * "Tahoma" but the resolution fails on some systems. */
+const char *
+gp_cairo_default_font(void)
+{
+#ifdef WIN32
+	return "Tahoma";
+#else
+	return "Sans";
+#endif
+}
+
 /*----------------------------------------------------------------------------
    coordinates functions
 ----------------------------------------------------------------------------*/
